
from __future__ import absolute_import
#typing

#overrides

from allennlp.common.util import JsonDict, sanitize
from allennlp.data import DatasetReader, Instance
from allennlp.models import Model
from allennlp.predictors.predictor import Predictor
from allennlp.data.tokenizers.word_splitter import SpacyWordSplitter

# POS tags have a unified colour.
NODE_TYPE_TO_STYLE = {}

NODE_TYPE_TO_STYLE[u"root"] = [u"color5", u"strong"]
NODE_TYPE_TO_STYLE[u"dep"] = [u"color5", u"strong"]

# Arguments
NODE_TYPE_TO_STYLE[u"nsubj"] = [u"color1"]
NODE_TYPE_TO_STYLE[u"nsubjpass"] = [u"color1"]
NODE_TYPE_TO_STYLE[u"csubj"] = [u"color1"]
NODE_TYPE_TO_STYLE[u"csubjpass"] = [u"color1"]

# Complements
NODE_TYPE_TO_STYLE[u"pobj"] = [u"color2"]
NODE_TYPE_TO_STYLE[u"dobj"] = [u"color2"]
NODE_TYPE_TO_STYLE[u"iobj"] = [u"color2"]
NODE_TYPE_TO_STYLE[u"mark"] = [u"color2"]
NODE_TYPE_TO_STYLE[u"pcomp"] = [u"color2"]
NODE_TYPE_TO_STYLE[u"xcomp"] = [u"color2"]
NODE_TYPE_TO_STYLE[u"ccomp"] = [u"color2"]
NODE_TYPE_TO_STYLE[u"acomp"] = [u"color2"]

# Modifiers
NODE_TYPE_TO_STYLE[u"aux"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"cop"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"det"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"conj"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"cc"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"prep"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"number"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"possesive"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"poss"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"discourse"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"expletive"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"prt"] = [u"color3"]
NODE_TYPE_TO_STYLE[u"advcl"] = [u"color3"]

NODE_TYPE_TO_STYLE[u"mod"] = [u"color4"]
NODE_TYPE_TO_STYLE[u"amod"] = [u"color4"]
NODE_TYPE_TO_STYLE[u"tmod"] = [u"color4"]
NODE_TYPE_TO_STYLE[u"quantmod"] = [u"color4"]
NODE_TYPE_TO_STYLE[u"npadvmod"] = [u"color4"]
NODE_TYPE_TO_STYLE[u"infmod"] = [u"color4"]
NODE_TYPE_TO_STYLE[u"advmod"] = [u"color4"]
NODE_TYPE_TO_STYLE[u"appos"] = [u"color4"]
NODE_TYPE_TO_STYLE[u"nn"] = [u"color4"]

NODE_TYPE_TO_STYLE[u"neg"] = [u"color0"]
NODE_TYPE_TO_STYLE[u"punct"] = [u"color0"]


LINK_TO_POSITION = {}
# Put subjects on the left
LINK_TO_POSITION[u"nsubj"] = u"left"
LINK_TO_POSITION[u"nsubjpass"] = u"left"
LINK_TO_POSITION[u"csubj"] = u"left"
LINK_TO_POSITION[u"csubjpass"] = u"left"

# Put arguments and some clauses on the right
LINK_TO_POSITION[u"pobj"] = u"right"
LINK_TO_POSITION[u"dobj"] = u"right"
LINK_TO_POSITION[u"iobj"] = u"right"
LINK_TO_POSITION[u"pcomp"] = u"right"
LINK_TO_POSITION[u"xcomp"] = u"right"
LINK_TO_POSITION[u"ccomp"] = u"right"
LINK_TO_POSITION[u"acomp"] = u"right"

class BiaffineDependencyParserPredictor(Predictor):
    u"""
    Predictor for the :class:`~allennlp.models.BiaffineDependencyParser` model.
    """
    def __init__(self, model       , dataset_reader               )        :
        super(BiaffineDependencyParserPredictor, self).__init__(model, dataset_reader)
        # TODO(Mark) Make the language configurable and based on a model attribute.
        self._tokenizer = SpacyWordSplitter(language=u'en_core_web_sm', pos_tags=True)

    def predict(self, sentence     )            :
        u"""
        Predict a dependency parse for the given sentence.
        Parameters
        ----------
        sentence The sentence to parse.

        Returns
        -------
        A dictionary representation of the dependency tree.
        """
        return self.predict_json({u"sentence" : sentence})

    #overrides
    def _json_to_instance(self, json_dict          )            :
        u"""
        Expects JSON that looks like ``{"sentence": "..."}``.
        """
        spacy_tokens = self._tokenizer.split_words(json_dict[u"sentence"])
        sentence_text = [token.text for token in spacy_tokens]
        pos_tags = [token.tag_ for token in spacy_tokens]
        return self._dataset_reader.text_to_instance(sentence_text, pos_tags)

    #overrides
    def predict_instance(self, instance          )            :
        outputs = self._model.forward_on_instance(instance)

        words = outputs[u"words"]
        pos = outputs[u"pos"]
        heads = outputs[u"predicted_heads"]
        tags = outputs[u"predicted_dependencies"]
        outputs[u"hierplane_tree"] = self._build_hierplane_tree(words, heads, tags, pos)
        return sanitize(outputs)

    #overrides
    def predict_batch_instance(self, instances                )                  :
        outputs = self._model.forward_on_instances(instances)
        for output in outputs:
            words = output[u"words"]
            pos = output[u"pos"]
            heads = output[u"predicted_heads"]
            tags = output[u"predicted_dependencies"]
            output[u"hierplane_tree"] = self._build_hierplane_tree(words, heads, tags, pos)
        return sanitize(outputs)

    @staticmethod
    def _build_hierplane_tree(words           ,
                              heads           ,
                              tags           ,
                              pos           )                  :
        u"""
        Returns
        -------
        A JSON dictionary render-able by Hierplane for the given tree.
        """

        word_index_to_cumulative_indices                             = {}
        cumulative_index = 0
        for i, word in enumerate(words):
            word_length = len(word) + 1
            word_index_to_cumulative_indices[i] = (cumulative_index, cumulative_index + word_length)
            cumulative_index += word_length

        def node_constuctor(index     ):
            children = []
            for next_index, child in enumerate(heads):
                if child == index + 1:
                    children.append(node_constuctor(next_index))

            # These are the icons which show up in the bottom right
            # corner of the node.
            attributes = [pos[index]]
            start, end = word_index_to_cumulative_indices[index]

            hierplane_node = {
                    u"word": words[index],
                    # The type of the node - all nodes with the same
                    # type have a unified colour.
                    u"nodeType": tags[index],
                    # Attributes of the node.
                    u"attributes": attributes,
                    # The link between  the node and it's parent.
                    u"link": tags[index],
                    u"spans": [{u"start": start, u"end": end}]
            }
            if children:
                hierplane_node[u"children"] = children
            return hierplane_node
        # We are guaranteed that there is a single word pointing to
        # the root index, so we can find it just by searching for 0 in the list.
        root_index = heads.index(0)
        hierplane_tree = {
                u"text": u" ".join(words),
                u"root": node_constuctor(root_index),
                u"nodeTypeToStyle": NODE_TYPE_TO_STYLE,
                u"linkToPosition": LINK_TO_POSITION
        }
        return hierplane_tree

BiaffineDependencyParserPredictor = Predictor.register(u'biaffine-dependency-parser')(BiaffineDependencyParserPredictor)



# pylint: disable=no-self-use,invalid-name

from __future__ import absolute_import
import numpy
import torch
from torch.nn.utils.rnn import pack_padded_sequence, pad_packed_sequence

from allennlp.modules.stacked_alternating_lstm import StackedAlternatingLstm
from allennlp.common.testing import AllenNlpTestCase


class TestStackedAlternatingLstm(AllenNlpTestCase):
    def test_stacked_alternating_lstm_completes_forward_pass(self):
        input_tensor = torch.rand(4, 5, 3)
        input_tensor[1, 4:, :] = 0.
        input_tensor[2, 2:, :] = 0.
        input_tensor[3, 1:, :] = 0.
        input_tensor = pack_padded_sequence(input_tensor, [5, 4, 2, 1], batch_first=True)
        lstm = StackedAlternatingLstm(3, 7, 3)
        output, _ = lstm(input_tensor)
        output_sequence, _ = pad_packed_sequence(output, batch_first=True)
        numpy.testing.assert_array_equal(output_sequence.data[1, 4:, :].numpy(), 0.0)
        numpy.testing.assert_array_equal(output_sequence.data[2, 2:, :].numpy(), 0.0)
        numpy.testing.assert_array_equal(output_sequence.data[3, 1:, :].numpy(), 0.0)

    def test_lstms_are_interleaved(self):
        lstm = StackedAlternatingLstm(3, 7, 8)
        for i, layer in enumerate(lstm.lstm_layers):
            if i % 2 == 0:
                assert layer.go_forward
            else:
                assert not layer.go_forward



# pylint: disable=invalid-name,no-self-use,protected-access

from __future__ import absolute_import
import torch
from torch.autograd import Variable

from allennlp.common.testing import AllenNlpTestCase
from allennlp.nn.decoding import ConstrainedBeamSearch
from .simple_transition_system import SimpleDecoderState, SimpleDecoderStep


class TestConstrainedBeamSearch(AllenNlpTestCase):
    def test_search(self):
        # The simple transition system starts at some number, adds one or two at each state, and
        # tries to get to 4.  The highest scoring path has the shortest length and the highest
        # numbers (so always add two, unless you're at 3).  From -3, there are lots of possible
        # sequences: [-2, -1, 0, 1, 2, 3, 4], [-1, 1, 3, 4], ...  We'll specify a few of those up
        # front as "allowed", and use that to test the constrained beam search implementation.
        initial_state = SimpleDecoderState([0],
                                           [[]],
                                           [Variable(torch.Tensor([0.0]))],
                                           [-3])
        beam_size = 3
        # pylint: disable=bad-whitespace
        allowed_sequences = Variable(torch.Tensor([[[-2, -1, 0, 1,  2,  3,  4],
                                                    [-2,  0, 2, 4, -1, -1, -1],
                                                    [-1,  1, 3, 4, -1, -1, -1],
                                                    [-2, -1, 0, 1,  2,  4, -1],
                                                    [-1,  0, 1, 2,  3,  4, -1],
                                                    [-1,  1, 2, 3,  4, -1, -1]]]))
        # pylint: enable=bad-whitespace
        mask = Variable(torch.Tensor([[[1, 1, 1, 1, 1, 1, 1],
                                       [1, 1, 1, 1, 0, 0, 0],
                                       [1, 1, 1, 1, 0, 0, 0],
                                       [1, 1, 1, 1, 1, 1, 0],
                                       [1, 1, 1, 1, 1, 1, 0],
                                       [1, 1, 1, 1, 1, 0, 0]]]))

        beam_search = ConstrainedBeamSearch(beam_size, allowed_sequences, mask)

        # Including the value in the score will make us pick states that have higher numbers first.
        # So with a beam size of 3, we'll get all of the states that start with `-1` after the
        # first step, even though in the end one of the states that starts with `-2` is better than
        # two of the states that start with `-1`.
        decoder_step = SimpleDecoderStep(include_value_in_score=True)
        best_states = beam_search.search(initial_state, decoder_step)

        assert len(best_states) == 1
        assert best_states[0][0].action_history[0] == [-1, 1, 3, 4]
        assert best_states[0][1].action_history[0] == [-1, 1, 2, 3, 4]
        assert best_states[0][2].action_history[0] == [-1, 0, 1, 2, 3, 4]

        # With a beam size of 6, we should get the other allowed path of length 4 as the second
        # best result.
        beam_size = 6
        beam_search = ConstrainedBeamSearch(beam_size, allowed_sequences, mask)
        decoder_step = SimpleDecoderStep(include_value_in_score=True)
        best_states = beam_search.search(initial_state, decoder_step)

        assert len(best_states) == 1
        assert best_states[0][0].action_history[0] == [-1, 1, 3, 4]
        assert best_states[0][1].action_history[0] == [-2, 0, 2, 4]



u"""
A :class:`~allennlp.training.metrics.metric.Metric` is some quantity or quantities
that can be accumulated during training or evaluation; for example,
accuracy or F1 score.
"""


from __future__ import absolute_import
# from allennlp.training.metrics.metric import Metric
# from allennlp.training.metrics.average import Average
# from allennlp.training.metrics.boolean_accuracy import BooleanAccuracy
# from allennlp.training.metrics.categorical_accuracy import CategoricalAccuracy
# from allennlp.training.metrics.conll_coref_scores import ConllCorefScores
# from allennlp.training.metrics.entropy import Entropy
# from allennlp.training.metrics.evalb_bracketing_scorer import EvalbBracketingScorer, DEFAULT_EVALB_DIR
# from allennlp.training.metrics.f1_measure import F1Measure
# from allennlp.training.metrics.mention_recall import MentionRecall
# from allennlp.training.metrics.span_based_f1_measure import SpanBasedF1Measure
# from allennlp.training.metrics.squad_em_and_f1 import SquadEmAndF1
# from allennlp.training.metrics.wikitables_accuracy import WikiTablesAccuracy
# from allennlp.training.metrics.attachment_scores import AttachmentScores

class Metric:
    pass

class Average:
    pass

class BooleanAccuracy:
    pass

class CategoricalAccuracy:
    pass

class ConllCorefScores:
    pass

class Entropy:
    pass

class EvalbBracketingScorer:
    pass

DEFAULT_EVALB_DIR = None

class F1Measure:
    pass

class MentionRecall:
    pass

class SpanBasedF1Measure:
    pass

class SquadEmAndF1:
    pass

class WikiTablesAccuracy:
    pass

class AttachmentScores:
    pass





from __future__ import absolute_import
from allennlp.semparse.worlds.nlvr_world import NlvrWorld
from allennlp.semparse.worlds.wikitables_world import WikiTablesWorld
from allennlp.semparse.worlds.atis_world import AtisWorld




from __future__ import absolute_import
from collections import defaultdict
#typing

import torch
try:
    from itertools import izip
except:
    izip = zip



def construct_prefix_tree(targets                                            ,
                          target_mask                         = None)                                         :
    u"""
    Takes a list of valid target action sequences and creates a mapping from all possible
    (valid) action prefixes to allowed actions given that prefix.  While the method is called
    ``construct_prefix_tree``, we're actually returning a map that has as keys the paths to
    `all internal nodes of the trie`, and as values all of the outgoing edges from that node.

    ``targets`` is assumed to be a tensor of shape ``(batch_size, num_valid_sequences,
    sequence_length)``.  If the mask is not ``None``, it is assumed to have the same shape, and
    we will ignore any value in ``targets`` that has a value of ``0`` in the corresponding
    position in the mask.  We assume that the mask has the format 1*0* for each item in
    ``targets`` - that is, once we see our first zero, we stop processing that target.

    For example, if ``targets`` is the following tensor: ``[[1, 2, 3], [1, 4, 5]]``, the return
    value will be: ``{(): set([1]), (1,): set([2, 4]), (1, 2): set([3]), (1, 4): set([5])}``.

    This could be used, e.g., to do an efficient constrained beam search, or to efficiently
    evaluate the probability of all of the target sequences.
    """
    batched_allowed_transitions                                        = []

    if not isinstance(targets, list):
        assert targets.dim() == 3, u"targets tensor needs to be batched!"
        targets = targets.detach().cpu().numpy().tolist()
    if target_mask is not None:
        target_mask = target_mask.detach().cpu().numpy().tolist()
    else:
        target_mask = [None for _ in targets]

    for instance_targets, instance_mask in izip(targets, target_mask):
        allowed_transitions                                  = defaultdict(set)
        for i, target_sequence in enumerate(instance_targets):
            history                  = ()
            for j, action in enumerate(target_sequence):
                if instance_mask and instance_mask[i][j] == 0:
                    break
                allowed_transitions[history].add(action)
                history = history + (action,)
        batched_allowed_transitions.append(allowed_transitions)
    return batched_allowed_transitions



# pylint: disable=no-self-use,invalid-name

from __future__ import absolute_import
from collections import defaultdict

from allennlp.common.testing import AllenNlpTestCase
from allennlp.data import Token, Vocabulary
from allennlp.data.token_indexers import DepLabelIndexer
from allennlp.data.tokenizers.word_splitter import SpacyWordSplitter


class TestDepLabelIndexer(AllenNlpTestCase):
    def setUp(self):
        super(TestDepLabelIndexer, self).setUp()
        self.tokenizer = SpacyWordSplitter(parse=True)

    def test_count_vocab_items_uses_pos_tags(self):
        tokens = self.tokenizer.split_words(u"This is a sentence.")
        tokens = [Token(u"<S>")] + [t for t in tokens] + [Token(u"</S>")]
        indexer = DepLabelIndexer()
        counter = defaultdict(lambda: defaultdict(int))
        for token in tokens:
            indexer.count_vocab_items(token, counter)

        assert counter[u"dep_labels"] == {u"ROOT": 1, u"nsubj": 1,
                                         u"det": 1, u"NONE": 2, u"attr": 1, u"punct": 1}

    def test_tokens_to_indices_uses_pos_tags(self):
        tokens = self.tokenizer.split_words(u"This is a sentence.")
        tokens = [t for t in tokens] + [Token(u"</S>")]
        vocab = Vocabulary()
        root_index = vocab.add_token_to_namespace(u'ROOT', namespace=u'dep_labels')
        none_index = vocab.add_token_to_namespace(u'NONE', namespace=u'dep_labels')
        indexer = DepLabelIndexer()
        assert indexer.tokens_to_indices([tokens[1]], vocab, u"tokens1") == {u"tokens1": [root_index]}
        assert indexer.tokens_to_indices([tokens[-1]], vocab, u"tokens-1") == {u"tokens-1": [none_index]}

    def test_padding_functions(self):
        indexer = DepLabelIndexer()
        assert indexer.get_padding_token() == 0
        assert indexer.get_padding_lengths(0) == {}

    def test_as_array_produces_token_sequence(self):
        indexer = DepLabelIndexer()
        padded_tokens = indexer.pad_token_sequence({u'key': [1, 2, 3, 4, 5]}, {u'key': 10}, {})
        assert padded_tokens == {u'key': [1, 2, 3, 4, 5, 0, 0, 0, 0, 0]}




from __future__ import absolute_import
#typing
import json

from allennlp.common import Registrable
from allennlp.common.checks import ConfigurationError
from allennlp.common.util import JsonDict, sanitize
from allennlp.data import DatasetReader, Instance
from allennlp.models import Model
from allennlp.models.archival import load_archive

# a mapping from model `type` to the default Predictor for that type
DEFAULT_PREDICTORS = {
        u'srl': u'semantic-role-labeling',
        u'decomposable_attention': u'textual-entailment',
        u'bidaf': u'machine-comprehension',
        u'bidaf-ensemble': u'machine-comprehension',
        u'simple_tagger': u'sentence-tagger',
        u'crf_tagger': u'sentence-tagger',
        u'coref': u'coreference-resolution',
        u'constituency_parser': u'constituency-parser',
}

class Predictor(Registrable):
    u"""
    a ``Predictor`` is a thin wrapper around an AllenNLP model that handles JSON -> JSON predictions
    that can be used for serving models through the web API or making predictions in bulk.
    """
    def __init__(self, model       , dataset_reader               )        :
        self._model = model
        self._dataset_reader = dataset_reader

    def load_line(self, line     )            :  # pylint: disable=no-self-use
        u"""
        If your inputs are not in JSON-lines format (e.g. you have a CSV)
        you can override this function to parse them correctly.
        """
        return json.loads(line)

    def dump_line(self, outputs          )       :  # pylint: disable=no-self-use
        u"""
        If you don't want your outputs in JSON-lines format
        you can override this function to output them differently.
        """
        return json.dumps(outputs) + u"\n"

    def predict_json(self, inputs          )            :
        instance = self._json_to_instance(inputs)
        return self.predict_instance(instance)

    def predict_instance(self, instance          )            :
        outputs = self._model.forward_on_instance(instance)
        return sanitize(outputs)

    def _json_to_instance(self, json_dict          )            :
        u"""
        Converts a JSON object into an :class:`~allennlp.data.instance.Instance`
        and a ``JsonDict`` of information which the ``Predictor`` should pass through,
        such as tokenised inputs.
        """
        raise NotImplementedError

    def predict_batch_json(self, inputs                )                  :
        instances = self._batch_json_to_instances(inputs)
        return self.predict_batch_instance(instances)

    def predict_batch_instance(self, instances                )                  :
        outputs = self._model.forward_on_instances(instances)
        return sanitize(outputs)

    def _batch_json_to_instances(self, json_dicts                )                  :
        u"""
        Converts a list of JSON objects into a list of :class:`~allennlp.data.instance.Instance`s.
        By default, this expects that a "batch" consists of a list of JSON blobs which would
        individually be predicted by :func:`predict_json`. In order to use this method for
        batch prediction, :func:`_json_to_instance` should be implemented by the subclass, or
        if the instances have some dependency on each other, this method should be overridden
        directly.
        """
        instances = []
        for json_dict in json_dicts:
            instances.append(self._json_to_instance(json_dict))
        return instances

    @classmethod
    def from_path(cls, archive_path     , predictor_name      = None)               :
        u"""
        Instantiate a :class:`Predictor` from an archive path.

        If you need more detailed configuration options, such as running the predictor on the GPU,
        please use `from_archive`.

        Parameters
        ----------
        archive_path The path to the archive.

        Returns
        -------
        A Predictor instance.
        """
        return Predictor.from_archive(load_archive(archive_path), predictor_name)

    @classmethod
    def from_archive(cls, archive         , predictor_name      = None)               :
        u"""
        Instantiate a :class:`Predictor` from an :class:`~allennlp.models.archival.Archive`;
        that is, from the result of training a model. Optionally specify which `Predictor`
        subclass; otherwise, the default one for the model will be used.
        """
        # Duplicate the config so that the config inside the archive doesn't get consumed
        config = archive.config.duplicate()

        if not predictor_name:
            model_type = config.get(u"model").get(u"type")
            if not model_type in DEFAULT_PREDICTORS:
                raise ConfigurationError("No default predictor for model type {model_type}.\n"\
                                         "Please specify a predictor explicitly.")
            predictor_name = DEFAULT_PREDICTORS[model_type]

        dataset_reader_params = config[u"dataset_reader"]
        dataset_reader = DatasetReader.from_params(dataset_reader_params)

        model = archive.model
        model.eval()

        return Predictor.by_name(predictor_name)(model, dataset_reader)



# pylint: disable=invalid-name,no-self-use,protected-access



from __future__ import with_statement
from __future__ import absolute_import
import numpy
import pytest
import torch

from allennlp.common.testing import AllenNlpTestCase
from allennlp.common.checks import ConfigurationError
from allennlp.nn.decoding.chu_liu_edmonds import _find_cycle, decode_mst

class ChuLiuEdmondsTest(AllenNlpTestCase):
    def test_find_cycle(self):
        # No cycle
        parents = [0, 2, 3, 0, 3]
        current_nodes = [True for _ in range(5)]
        has_cycle, cycle = _find_cycle(parents, 5, current_nodes)
        assert not has_cycle
        assert not cycle

        # Cycle
        parents = [0, 2, 3, 1, 3]
        has_cycle, cycle = _find_cycle(parents, 5, current_nodes)
        assert has_cycle
        assert cycle == [1, 2, 3]

        # No cycle if ignored nodes are correctly ignored.
        parents = [-1, 0, 1, 4, 3]
        current_nodes = [True for _ in range(5)]
        current_nodes[4] = False
        current_nodes[3] = False
        has_cycle, cycle = _find_cycle(parents, 5, current_nodes)
        assert not has_cycle
        assert cycle == []

        # Cycle, but excluding ignored nodes which form their own cycle.
        parents = [-1, 2, 1, 4, 3]
        current_nodes = [True for _ in range(5)]
        current_nodes[1] = False
        current_nodes[2] = False
        has_cycle, cycle = _find_cycle(parents, 5, current_nodes)
        assert has_cycle
        assert cycle == [3, 4]

    def test_mst(self):
        # First, test some random cases as sanity checks.
        # No label case
        energy = numpy.random.rand(5, 5)
        heads, types = decode_mst(energy, 5, has_labels=False)
        assert not _find_cycle(heads, 5, [True] * 5)[0]

        # Labeled case
        energy = numpy.random.rand(3, 5, 5)
        heads, types = decode_mst(energy, 5)

        assert not _find_cycle(heads, 5, [True] * 5)[0]
        label_id_matrix = energy.argmax(axis=0)

        # Check that the labels correspond to the
        # argmax of the labels for the arcs.
        for child, parent in enumerate(heads):
            # The first index corresponds to the symbolic
            # head token, which won't necessarily have an
            # argmax type.
            if child == 0:
                continue
            assert types[child] == label_id_matrix[parent, child]

        # Check wrong dimensions throw errors
        with pytest.raises(ConfigurationError):
            energy = numpy.random.rand(5, 5)
            decode_mst(energy, 5, has_labels=True)

        with pytest.raises(ConfigurationError):
            energy = numpy.random.rand(3, 5, 5)
            decode_mst(energy, 5, has_labels=False)

    def test_mst_finds_maximum_spanning_tree(self):
        energy = torch.range(1, 9).view(1, 3, 3)
        heads, _ = decode_mst(energy.numpy(), 3) # pylint: disable=protected-access
        assert list(heads) == [-1, 2, 0]





from __future__ import absolute_import
#overrides
from allennlp.common.util import JsonDict
from allennlp.data import Instance
from allennlp.predictors.predictor import Predictor


class DecomposableAttentionPredictor(Predictor):
    u"""
    Predictor for the :class:`~allennlp.models.bidaf.DecomposableAttention` model.
    """

    def predict(self, premise     , hypothesis     )            :
        u"""
        Predicts whether the hypothesis is entailed by the premise text.

        Parameters
        ----------
        premise : ``str``
            A passage representing what is assumed to be true.

        hypothesis : ``str``
            A sentence that may be entailed by the premise.

        Returns
        -------
        A dictionary where the key "label_probs" determines the probabilities of each of
        [entailment, contradiction, neutral].
        """
        return self.predict_json({u"premise" : premise, u"hypothesis": hypothesis})

    #overrides
    def _json_to_instance(self, json_dict          )            :
        u"""
        Expects JSON that looks like ``{"premise": "...", "hypothesis": "..."}``.
        """
        premise_text = json_dict[u"premise"]
        hypothesis_text = json_dict[u"hypothesis"]
        return self._dataset_reader.text_to_instance(premise_text, hypothesis_text)

DecomposableAttentionPredictor = Predictor.register(u'textual-entailment')(DecomposableAttentionPredictor)



# pylint: disable=no-self-use,protected-access


from __future__ import division
from __future__ import absolute_import
import torch
import numpy as np
from numpy.testing import assert_almost_equal

from allennlp.common.testing import AllenNlpTestCase
from allennlp.nn.decoding.decoder_trainers import ExpectedRiskMinimization
from ..simple_transition_system import SimpleDecoderState, SimpleDecoderStep


class TestExpectedRiskMinimization(AllenNlpTestCase):
    def setUp(self):
        super(TestExpectedRiskMinimization, self).setUp()
        self.initial_state = SimpleDecoderState([0], [[0]], [torch.Tensor([0.0])])
        self.decoder_step = SimpleDecoderStep()
        # Cost is the number of odd elements in the action history.
        self.supervision = lambda state: torch.Tensor([sum([x%2 != 0 for x in
                                                            state.action_history[0]])])
        # High beam size ensures exhaustive search.
        self.trainer = ExpectedRiskMinimization(beam_size=100,
                                                normalize_by_length=False,
                                                max_decoding_steps=10)

    def test_get_finished_states(self):
        finished_states = self.trainer._get_finished_states(self.initial_state, self.decoder_step)
        state_info = [(state.action_history[0], state.score[0].item()) for state in finished_states]
        # There will be exactly five finished states with the following paths. Each score is the
        # negative of one less than the number of elements in the action history.
        assert len(finished_states) == 5
        assert ([0, 2, 4], -2) in state_info
        assert ([0, 1, 2, 4], -3) in state_info
        assert ([0, 1, 3, 4], -3) in state_info
        assert ([0, 2, 3, 4], -3) in state_info
        assert ([0, 1, 2, 3, 4], -4) in state_info

    def test_decode(self):
        decoded_info = self.trainer.decode(self.initial_state, self.decoder_step, self.supervision)
        # The best state corresponds to the shortest path.
        assert decoded_info[u'best_action_sequences'][0] == [[0, 2, 4]]
        # The scores and costs corresponding to the finished states will be
        # [0, 2, 4] : -2, 0
        # [0, 1, 2, 4] : -3, 1
        # [0, 1, 3, 4] : -3, 2
        # [0, 2, 3, 4] : -3, 1
        # [0, 1, 2, 3, 4] : -4, 2

        # This is the normalization factor while re-normalizing probabilities on the beam
        partition = np.exp(-2) + np.exp(-3) + np.exp(-3) + np.exp(-3) + np.exp(-4)
        expected_loss = ((np.exp(-2) * 0) + (np.exp(-3) * 1) + (np.exp(-3) * 2) +
                         (np.exp(-3) * 1) + (np.exp(-4) * 2)) / partition
        assert_almost_equal(decoded_info[u'loss'].data.numpy(), expected_loss)




from __future__ import absolute_import
#typing
#overrides

from nltk.sem.logic import TRUTH_TYPE, BasicType, EntityType, Type

from allennlp.semparse.type_declarations.type_declaration import ComplexType, HigherOrderType, NamedBasicType


class NegateFilterType(HigherOrderType):
    u"""
    Because our negate filters are higher-order functions, we need to make an explicit class here,
    to make sure that we've overridden the right methods correctly.
    """
    def __init__(self, first, second):
        super(NegateFilterType, self).__init__(num_arguments=1, first=first, second=second)

    #overrides
    def substitute_any_type(self, basic_types                )              :
        # There's no ANY_TYPE in here, so we don't need to do any substitution.
        return [self]


# All constants default to ``EntityType`` in NLTK. For domains where constants of different types
# appear in the logical forms, we have a way of specifying ``constant_type_prefixes`` and passing
# them to the constructor of ``World``. However, in the NLVR language we defined, we see constants
# of just one type, number. So we let them default to ``EntityType``.
NUM_TYPE = EntityType()
BOX_TYPE = NamedBasicType(u"BOX")
OBJECT_TYPE = NamedBasicType(u"OBJECT")
COLOR_TYPE = NamedBasicType(u"COLOR")
SHAPE_TYPE = NamedBasicType(u"SHAPE")
OBJECT_FILTER_TYPE = ComplexType(OBJECT_TYPE, OBJECT_TYPE)
NEGATE_FILTER_TYPE = NegateFilterType(ComplexType(OBJECT_TYPE, OBJECT_TYPE),
                                      ComplexType(OBJECT_TYPE, OBJECT_TYPE))
BOX_MEMBERSHIP_TYPE = ComplexType(BOX_TYPE, OBJECT_TYPE)

BOX_COLOR_FILTER_TYPE = ComplexType(BOX_TYPE, ComplexType(COLOR_TYPE, BOX_TYPE))
BOX_SHAPE_FILTER_TYPE = ComplexType(BOX_TYPE, ComplexType(SHAPE_TYPE, BOX_TYPE))
BOX_COUNT_FILTER_TYPE = ComplexType(BOX_TYPE, ComplexType(NUM_TYPE, BOX_TYPE))
# This box filter returns boxes where a specified attribute is same or different
BOX_ATTRIBUTE_SAME_FILTER_TYPE = ComplexType(BOX_TYPE, BOX_TYPE)


ASSERT_COLOR_TYPE = ComplexType(OBJECT_TYPE, ComplexType(COLOR_TYPE, TRUTH_TYPE))
ASSERT_SHAPE_TYPE = ComplexType(OBJECT_TYPE, ComplexType(SHAPE_TYPE, TRUTH_TYPE))
ASSERT_BOX_COUNT_TYPE = ComplexType(BOX_TYPE, ComplexType(NUM_TYPE, TRUTH_TYPE))
ASSERT_OBJECT_COUNT_TYPE = ComplexType(OBJECT_TYPE, ComplexType(NUM_TYPE, TRUTH_TYPE))

BOX_EXISTS_TYPE = ComplexType(BOX_TYPE, TRUTH_TYPE)
OBJECT_EXISTS_TYPE = ComplexType(OBJECT_TYPE, TRUTH_TYPE)


COMMON_NAME_MAPPING = {}
COMMON_TYPE_SIGNATURE = {}

BASIC_TYPES = set([NUM_TYPE, BOX_TYPE, OBJECT_TYPE, COLOR_TYPE, SHAPE_TYPE])


def add_common_name_with_type(name, mapping, type_signature):
    COMMON_NAME_MAPPING[name] = mapping
    COMMON_TYPE_SIGNATURE[mapping] = type_signature


# Entities
add_common_name_with_type(u"all_objects", u"O", OBJECT_TYPE)
add_common_name_with_type(u"all_boxes", u"B", BOX_TYPE)
add_common_name_with_type(u"color_black", u"C0", COLOR_TYPE)
add_common_name_with_type(u"color_blue", u"C1", COLOR_TYPE)
add_common_name_with_type(u"color_yellow", u"C2", COLOR_TYPE)
add_common_name_with_type(u"shape_triangle", u"S0", SHAPE_TYPE)
add_common_name_with_type(u"shape_square", u"S1", SHAPE_TYPE)
add_common_name_with_type(u"shape_circle", u"S2", SHAPE_TYPE)


# Attribute function
add_common_name_with_type(u"object_in_box", u"I", BOX_MEMBERSHIP_TYPE)


# Assert functions
add_common_name_with_type(u"object_color_all_equals", u"A0", ASSERT_COLOR_TYPE)
add_common_name_with_type(u"object_color_any_equals", u"A28", ASSERT_COLOR_TYPE)
add_common_name_with_type(u"object_color_none_equals", u"A1", ASSERT_COLOR_TYPE)
add_common_name_with_type(u"object_shape_all_equals", u"A2", ASSERT_SHAPE_TYPE)
add_common_name_with_type(u"object_shape_any_equals", u"A29", ASSERT_SHAPE_TYPE)
add_common_name_with_type(u"object_shape_none_equals", u"A3", ASSERT_SHAPE_TYPE)
add_common_name_with_type(u"box_count_equals", u"A4", ASSERT_BOX_COUNT_TYPE)
add_common_name_with_type(u"box_count_not_equals", u"A5", ASSERT_BOX_COUNT_TYPE)
add_common_name_with_type(u"box_count_greater", u"A6", ASSERT_BOX_COUNT_TYPE)
add_common_name_with_type(u"box_count_greater_equals", u"A7", ASSERT_BOX_COUNT_TYPE)
add_common_name_with_type(u"box_count_lesser", u"A8", ASSERT_BOX_COUNT_TYPE)
add_common_name_with_type(u"box_count_lesser_equals", u"A9", ASSERT_BOX_COUNT_TYPE)
add_common_name_with_type(u"object_count_equals", u"A10", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_count_not_equals", u"A11", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_count_greater", u"A12", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_count_greater_equals", u"A13", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_count_lesser", u"A14", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_count_lesser_equals", u"A15", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_color_count_equals", u"A16", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_color_count_not_equals", u"A17", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_color_count_greater", u"A18", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_color_count_greater_equals", u"A19", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_color_count_lesser", u"A20", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_color_count_lesser_equals", u"A21", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_shape_count_equals", u"A22", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_shape_count_not_equals", u"A23", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_shape_count_greater", u"A24", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_shape_count_greater_equals", u"A25", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_shape_count_lesser", u"A26", ASSERT_OBJECT_COUNT_TYPE)
add_common_name_with_type(u"object_shape_count_lesser_equals", u"A27", ASSERT_OBJECT_COUNT_TYPE)

add_common_name_with_type(u"box_exists", u"E0", BOX_EXISTS_TYPE)
add_common_name_with_type(u"object_exists", u"E1", OBJECT_EXISTS_TYPE)


# Box filter functions
add_common_name_with_type(u"member_count_equals", u"F0", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_count_not_equals", u"F1", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_shape_all_equals", u"F2", BOX_SHAPE_FILTER_TYPE)
add_common_name_with_type(u"member_shape_any_equals", u"F26", BOX_SHAPE_FILTER_TYPE)
add_common_name_with_type(u"member_shape_none_equals", u"F3", BOX_SHAPE_FILTER_TYPE)
add_common_name_with_type(u"member_color_all_equals", u"F4", BOX_COLOR_FILTER_TYPE)
add_common_name_with_type(u"member_color_any_equals", u"F27", BOX_COLOR_FILTER_TYPE)
add_common_name_with_type(u"member_color_none_equals", u"F5", BOX_COLOR_FILTER_TYPE)
add_common_name_with_type(u"member_count_greater", u"F6", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_count_greater_equals", u"F7", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_count_lesser", u"F8", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_count_lesser_equals", u"F9", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_color_count_equals", u"F10", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_color_count_not_equals", u"F11", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_color_count_greater", u"F12", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_color_count_greater_equals", u"F13", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_color_count_lesser", u"F14", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_color_count_lesser_equals", u"F15", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_shape_count_equals", u"F16", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_shape_count_not_equals", u"F17", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_shape_count_greater", u"F18", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_shape_count_greater_equals", u"F19", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_shape_count_lesser", u"F20", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_shape_count_lesser_equals", u"F21", BOX_COUNT_FILTER_TYPE)
add_common_name_with_type(u"member_shape_same", u"F22", BOX_ATTRIBUTE_SAME_FILTER_TYPE)
add_common_name_with_type(u"member_color_same", u"F23", BOX_ATTRIBUTE_SAME_FILTER_TYPE)
add_common_name_with_type(u"member_shape_different", u"F24", BOX_ATTRIBUTE_SAME_FILTER_TYPE)
add_common_name_with_type(u"member_color_different", u"F25", BOX_ATTRIBUTE_SAME_FILTER_TYPE)


# Object filter functions
add_common_name_with_type(u"black", u"C3", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"blue", u"C4", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"yellow", u"C5", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"same_color", u"C6", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"triangle", u"S3", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"square", u"S4", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"circle", u"S5", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"same_shape", u"S6", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"touch_wall", u"T0", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"touch_corner", u"T1", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"touch_top", u"T2", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"touch_bottom", u"T3", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"touch_left", u"T4", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"touch_right", u"T5", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"touch_object", u"T6", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"above", u"L0", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"below", u"L1", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"top", u"L2", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"bottom", u"L3", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"small", u"Z0", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"medium", u"Z1", OBJECT_FILTER_TYPE)
add_common_name_with_type(u"big", u"Z2", OBJECT_FILTER_TYPE)

add_common_name_with_type(u"negate_filter", u"N", NEGATE_FILTER_TYPE)

# Adding numbers because they commonly occur in utterances. They're usually between 1 and 9. Since
# there are not too many of these productions, we're adding them to the global mapping instead of a
# local mapping in each world.
for num in range(1, 10):
    num_string = unicode(num)
    add_common_name_with_type(num_string, num_string, NUM_TYPE)





from __future__ import with_statement
from __future__ import absolute_import
import json
import logging
#typing

#overrides

from allennlp.common.file_utils import cached_path
from allennlp.data.dataset_readers.dataset_reader import DatasetReader
from allennlp.data.instance import Instance
from allennlp.data.dataset_readers.reading_comprehension import util
from allennlp.data.token_indexers import SingleIdTokenIndexer, TokenIndexer
from allennlp.data.tokenizers import Token, Tokenizer, WordTokenizer
from io import open
try:
    from itertools import izip
except:
    izip = zip


logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


class SquadReader(DatasetReader):
    u"""
    Reads a JSON-formatted SQuAD file and returns a ``Dataset`` where the ``Instances`` have four
    fields: ``question``, a ``TextField``, ``passage``, another ``TextField``, and ``span_start``
    and ``span_end``, both ``IndexFields`` into the ``passage`` ``TextField``.  We also add a
    ``MetadataField`` that stores the instance's ID, the original passage text, gold answer strings,
    and token offsets into the original passage, accessible as ``metadata['id']``,
    ``metadata['original_passage']``, ``metadata['answer_texts']`` and
    ``metadata['token_offsets']``.  This is so that we can more easily use the official SQuAD
    evaluation script to get metrics.

    Parameters
    ----------
    tokenizer : ``Tokenizer``, optional (default=``WordTokenizer()``)
        We use this ``Tokenizer`` for both the question and the passage.  See :class:`Tokenizer`.
        Default is ```WordTokenizer()``.
    token_indexers : ``Dict[str, TokenIndexer]``, optional
        We similarly use this for both the question and the passage.  See :class:`TokenIndexer`.
        Default is ``{"tokens": SingleIdTokenIndexer()}``.
    """
    def __init__(self,
                 tokenizer            = None,
                 token_indexers                          = None,
                 lazy       = False)        :
        super(SquadReader, self).__init__(lazy)
        self._tokenizer = tokenizer or WordTokenizer()
        self._token_indexers = token_indexers or {u'tokens': SingleIdTokenIndexer()}

    #overrides
    def _read(self, file_path     ):
        # if `file_path` is a URL, redirect to the cache
        file_path = cached_path(file_path)

        logger.info(u"Reading file at %s", file_path)
        with open(file_path) as dataset_file:
            dataset_json = json.load(dataset_file)
            dataset = dataset_json[u'data']
        logger.info(u"Reading the dataset")
        for article in dataset:
            for paragraph_json in article[u'paragraphs']:
                paragraph = paragraph_json[u"context"]
                tokenized_paragraph = self._tokenizer.tokenize(paragraph)

                for question_answer in paragraph_json[u'qas']:
                    question_text = question_answer[u"question"].strip().replace(u"\n", u"")
                    answer_texts = [answer[u'text'] for answer in question_answer[u'answers']]
                    span_starts = [answer[u'answer_start'] for answer in question_answer[u'answers']]
                    span_ends = [start + len(answer) for start, answer in izip(span_starts, answer_texts)]
                    instance = self.text_to_instance(question_text,
                                                     paragraph,
                                                     izip(span_starts, span_ends),
                                                     answer_texts,
                                                     tokenized_paragraph)
                    yield instance

    #overrides
    def text_to_instance(self,  # type: ignore
                         question_text     ,
                         passage_text     ,
                         char_spans                        = None,
                         answer_texts            = None,
                         passage_tokens              = None)            :
        # pylint: disable=arguments-differ
        if not passage_tokens:
            passage_tokens = self._tokenizer.tokenize(passage_text)
        char_spans = char_spans or []

        # We need to convert character indices in `passage_text` to token indices in
        # `passage_tokens`, as the latter is what we'll actually use for supervision.
        token_spans                        = []
        passage_offsets = [(token.idx, token.idx + len(token.text)) for token in passage_tokens]
        for char_span_start, char_span_end in char_spans:
            (span_start, span_end), error = util.char_span_to_token_span(passage_offsets,
                                                                         (char_span_start, char_span_end))
            if error:
                logger.debug(u"Passage: %s", passage_text)
                logger.debug(u"Passage tokens: %s", passage_tokens)
                logger.debug(u"Question text: %s", question_text)
                logger.debug(u"Answer span: (%d, %d)", char_span_start, char_span_end)
                logger.debug(u"Token span: (%d, %d)", span_start, span_end)
                logger.debug(u"Tokens in answer: %s", passage_tokens[span_start:span_end + 1])
                logger.debug(u"Answer: %s", passage_text[char_span_start:char_span_end])
            token_spans.append((span_start, span_end))

        return util.make_reading_comprehension_instance(self._tokenizer.tokenize(question_text),
                                                        passage_tokens,
                                                        self._token_indexers,
                                                        passage_text,
                                                        token_spans,
                                                        answer_texts)

SquadReader = DatasetReader.register(u"squad")(SquadReader)



# pylint: disable=no-self-use,invalid-name


from __future__ import absolute_import
from allennlp.common import Params
from allennlp.common.testing import AllenNlpTestCase
from allennlp.data.tokenizers import WordTokenizer
try:
    from itertools import izip
except:
    izip = zip


class TestWordTokenizer(AllenNlpTestCase):
    def test_passes_through_correctly(self):
        tokenizer = WordTokenizer(start_tokens=[u'@@', u'%%'], end_tokens=[u'^^'])
        sentence = u"this (sentence) has 'crazy' \"punctuation\"."
        tokens = [t.text for t in tokenizer.tokenize(sentence)]
        expected_tokens = [u"@@", u"%%", u"this", u"(", u"sentence", u")", u"has", u"'", u"crazy", u"'", u"\"",
                           u"punctuation", u"\"", u".", u"^^"]
        assert tokens == expected_tokens

    def test_batch_tokenization(self):
        tokenizer = WordTokenizer()
        sentences = [u"This is a sentence",
                     u"This isn't a sentence.",
                     u"This is the 3rd sentence."
                     u"Here's the 'fourth' sentence."]
        batch_tokenized = tokenizer.batch_tokenize(sentences)
        separately_tokenized = [tokenizer.tokenize(sentence) for sentence in sentences]
        assert len(batch_tokenized) == len(separately_tokenized)
        for batch_sentence, separate_sentence in izip(batch_tokenized, separately_tokenized):
            assert len(batch_sentence) == len(separate_sentence)
            for batch_word, separate_word in izip(batch_sentence, separate_sentence):
                assert batch_word.text == separate_word.text

    def test_stems_and_filters_correctly(self):
        tokenizer = WordTokenizer.from_params(Params({u'word_stemmer': {u'type': u'porter'},
                                                      u'word_filter': {u'type': u'stopwords'}}))
        sentence = u"this (sentence) has 'crazy' \"punctuation\"."
        expected_tokens = [u"sentenc", u"ha", u"crazi", u"punctuat"]
        tokens = [t.text for t in tokenizer.tokenize(sentence)]
        assert tokens == expected_tokens



# pylint: disable=no-self-use,invalid-name

from __future__ import absolute_import
import numpy
from numpy.testing import assert_almost_equal
import torch

from allennlp.common import Params
from allennlp.modules.seq2vec_encoders import CnnEncoder
from allennlp.nn import InitializerApplicator
from allennlp.common.testing import AllenNlpTestCase


class TestCnnEncoder(AllenNlpTestCase):
    def test_get_dimension_is_correct(self):
        encoder = CnnEncoder(embedding_dim=5, num_filters=4, ngram_filter_sizes=(3, 5))
        assert encoder.get_output_dim() == 8
        assert encoder.get_input_dim() == 5
        encoder = CnnEncoder(embedding_dim=5, num_filters=4, ngram_filter_sizes=(3, 5), output_dim=7)
        assert encoder.get_output_dim() == 7
        assert encoder.get_input_dim() == 5

    def test_can_construct_from_params(self):
        params = Params({
                u'embedding_dim': 5,
                u'num_filters': 4,
                u'ngram_filter_sizes': [3, 5]
                })
        encoder = CnnEncoder.from_params(params)
        assert encoder.get_output_dim() == 8
        params = Params({
                u'embedding_dim': 5,
                u'num_filters': 4,
                u'ngram_filter_sizes': [3, 5],
                u'output_dim': 7
                })
        encoder = CnnEncoder.from_params(params)
        assert encoder.get_output_dim() == 7

    def test_forward_does_correct_computation(self):
        encoder = CnnEncoder(embedding_dim=2, num_filters=1, ngram_filter_sizes=(1, 2))
        constant_init = lambda tensor: torch.nn.init.constant_(tensor, 1.)
        initializer = InitializerApplicator([(u".*", constant_init)])
        initializer(encoder)
        input_tensor = torch.FloatTensor([[[.7, .8], [.1, 1.5]]])
        encoder_output = encoder(input_tensor, None)
        assert_almost_equal(encoder_output.data.numpy(),
                            numpy.asarray([[1.6 + 1.0, 3.1 + 1.0]]),
                            decimal=6)

    def test_forward_runs_with_larger_input(self):
        encoder = CnnEncoder(embedding_dim=7,
                             num_filters=13,
                             ngram_filter_sizes=(1, 2, 3, 4, 5),
                             output_dim=30)
        tensor = torch.rand(4, 8, 7)
        assert encoder(tensor, None).size() == (4, 30)





from __future__ import with_statement
from __future__ import absolute_import
#typing
import itertools
import logging

#overrides

from allennlp.common.checks import ConfigurationError
from allennlp.common.file_utils import cached_path
from allennlp.data.dataset_readers.dataset_reader import DatasetReader
from allennlp.data.dataset_readers.dataset_utils import to_bioul
from allennlp.data.fields import TextField, SequenceLabelField, Field, MetadataField
from allennlp.data.instance import Instance
from allennlp.data.token_indexers import TokenIndexer, SingleIdTokenIndexer
from allennlp.data.tokenizers import Token
from io import open
try:
    from itertools import izip
except:
    izip = zip


logger = logging.getLogger(__name__)  # pylint: disable=invalid-name

def _is_divider(line     )        :
    empty_line = line.strip() == u''
    if empty_line:
        return True
    else:
        first_token = line.split()[0]
        if first_token == u"-DOCSTART-":  # pylint: disable=simplifiable-if-statement
            return True
        else:
            return False

_VALID_LABELS = set([u'ner', u'pos', u'chunk'])


class Conll2003DatasetReader(DatasetReader):
    u"""
    Reads instances from a pretokenised file where each line is in the following format:

    WORD POS-TAG CHUNK-TAG NER-TAG

    with a blank line indicating the end of each sentence
    and '-DOCSTART- -X- -X- O' indicating the end of each article,
    and converts it into a ``Dataset`` suitable for sequence tagging.

    Each ``Instance`` contains the words in the ``"tokens"`` ``TextField``.
    The values corresponding to the ``tag_label``
    values will get loaded into the ``"tags"`` ``SequenceLabelField``.
    And if you specify any ``feature_labels`` (you probably shouldn't),
    the corresponding values will get loaded into their own ``SequenceLabelField`` s.

    This dataset reader ignores the "article" divisions and simply treats
    each sentence as an independent ``Instance``. (Technically the reader splits sentences
    on any combination of blank lines and "DOCSTART" tags; in particular, it does the right
    thing on well formed inputs.)

    Parameters
    ----------
    token_indexers : ``Dict[str, TokenIndexer]``, optional (default=``{"tokens": SingleIdTokenIndexer()}``)
        We use this to define the input representation for the text.  See :class:`TokenIndexer`.
    tag_label: ``str``, optional (default=``ner``)
        Specify `ner`, `pos`, or `chunk` to have that tag loaded into the instance field `tag`.
    feature_labels: ``Sequence[str]``, optional (default=``()``)
        These labels will be loaded as features into the corresponding instance fields:
        ``pos`` -> ``pos_tags``, ``chunk`` -> ``chunk_tags``, ``ner`` -> ``ner_tags``
        Each will have its own namespace: ``pos_labels``, ``chunk_labels``, ``ner_labels``.
        If you want to use one of the labels as a `feature` in your model, it should be
        specified here.
    coding_scheme: ``str``, optional (default=``IOB1``)
        Specifies the coding scheme for ``ner_labels`` and ``chunk_labels``.
        Valid options are ``IOB1`` and ``BIOUL``.  The ``IOB1`` default maintains
        the original IOB1 scheme in the CoNLL data.
        In the IOB1 scheme, I is a token inside a span, O is a token outside
        a span and B is the beginning of span immediately following another
        span of the same type.
    """
    def __init__(self,
                 token_indexers                          = None,
                 tag_label      = u"ner",
                 feature_labels                = (),
                 lazy       = False,
                 coding_scheme      = u"IOB1")        :
        super(Conll2003DatasetReader, self).__init__(lazy)
        self._token_indexers = token_indexers or {u'tokens': SingleIdTokenIndexer()}
        if tag_label is not None and tag_label not in _VALID_LABELS:
            raise ConfigurationError(u"unknown tag label type: {}".format(tag_label))
        for label in feature_labels:
            if label not in _VALID_LABELS:
                raise ConfigurationError(u"unknown feature label type: {}".format(label))
        if coding_scheme not in (u"IOB1", u"BIOUL"):
            raise ConfigurationError(u"unknown coding_scheme: {}".format(coding_scheme))

        self.tag_label = tag_label
        self.feature_labels = set(feature_labels)
        self.coding_scheme = coding_scheme

    #overrides
    def _read(self, file_path     )                      :
        # if `file_path` is a URL, redirect to the cache
        file_path = cached_path(file_path)

        with open(file_path, u"r") as data_file:
            logger.info(u"Reading instances from lines in file at: %s", file_path)

            # Group into alternative divider / sentence chunks.
            for is_divider, lines in itertools.groupby(data_file, _is_divider):
                # Ignore the divider chunks, so that `lines` corresponds to the words
                # of a single sentence.
                if not is_divider:
                    fields = [line.strip().split() for line in lines]
                    # unzipping trick returns tuples, but our Fields need lists
                    tokens, pos_tags, chunk_tags, ner_tags = [list(field) for field in izip(*fields)]
                    # TextField requires ``Token`` objects
                    tokens = [Token(token) for token in tokens]

                    yield self.text_to_instance(tokens, pos_tags, chunk_tags, ner_tags)

    def text_to_instance(self, # type: ignore
                         tokens             ,
                         pos_tags            = None,
                         chunk_tags            = None,
                         ner_tags            = None)            :
        u"""
        We take `pre-tokenized` input here, because we don't have a tokenizer in this class.
        """
        # pylint: disable=arguments-differ
        sequence = TextField(tokens, self._token_indexers)
        instance_fields                   = {u'tokens': sequence}
        instance_fields[u"metadata"] = MetadataField({u"words": [x.text for x in tokens]})

        # Recode the labels if necessary.
        if self.coding_scheme == u"BIOUL":
            coded_chunks = to_bioul(chunk_tags) if chunk_tags is not None else None
            coded_ner = to_bioul(ner_tags) if ner_tags is not None else None
        else:
            # the default IOB1
            coded_chunks = chunk_tags
            coded_ner = ner_tags

        # Add "feature labels" to instance
        if u'pos' in self.feature_labels:
            if pos_tags is None:
                raise ConfigurationError(u"Dataset reader was specified to use pos_tags as "
                                         u"features. Pass them to text_to_instance.")
            instance_fields[u'pos_tags'] = SequenceLabelField(pos_tags, sequence, u"pos_tags")
        if u'chunk' in self.feature_labels:
            if coded_chunks is None:
                raise ConfigurationError(u"Dataset reader was specified to use chunk tags as "
                                         u"features. Pass them to text_to_instance.")
            instance_fields[u'chunk_tags'] = SequenceLabelField(coded_chunks, sequence, u"chunk_tags")
        if u'ner' in self.feature_labels:
            if coded_ner is None:
                raise ConfigurationError(u"Dataset reader was specified to use NER tags as "
                                         u" features. Pass them to text_to_instance.")
            instance_fields[u'ner_tags'] = SequenceLabelField(coded_ner, sequence, u"ner_tags")

        # Add "tag label" to instance
        if self.tag_label == u'ner' and coded_ner is not None:
            instance_fields[u'tags'] = SequenceLabelField(coded_ner, sequence)
        elif self.tag_label == u'pos' and pos_tags is not None:
            instance_fields[u'tags'] = SequenceLabelField(pos_tags, sequence)
        elif self.tag_label == u'chunk' and coded_chunks is not None:
            instance_fields[u'tags'] = SequenceLabelField(coded_chunks, sequence)

        return Instance(instance_fields)

Conll2003DatasetReader = DatasetReader.register(u"conll2003")(Conll2003DatasetReader)



# pylint: disable=no-self-use,invalid-name

from __future__ import absolute_import
from collections import defaultdict

from allennlp.common.testing import AllenNlpTestCase
from allennlp.data import Token, Vocabulary
from allennlp.data.token_indexers import PosTagIndexer
from allennlp.data.tokenizers.word_splitter import SpacyWordSplitter


class TestPosTagIndexer(AllenNlpTestCase):
    def setUp(self):
        super(TestPosTagIndexer, self).setUp()
        self.tokenizer = SpacyWordSplitter(pos_tags=True)

    def test_count_vocab_items_uses_pos_tags(self):
        tokens = self.tokenizer.split_words(u"This is a sentence.")
        tokens = [Token(u"<S>")] + [t for t in tokens] + [Token(u"</S>")]
        indexer = PosTagIndexer()
        counter = defaultdict(lambda: defaultdict(int))
        for token in tokens:
            indexer.count_vocab_items(token, counter)
        assert counter[u"pos_tags"] == {u'DT': 2, u'VBZ': 1, u'.': 1, u'NN': 1, u'NONE': 2}

        indexer._coarse_tags = True  # pylint: disable=protected-access
        counter = defaultdict(lambda: defaultdict(int))
        for token in tokens:
            indexer.count_vocab_items(token, counter)
        assert counter[u"pos_tags"] == {u'VERB': 1, u'PUNCT': 1, u'DET': 2, u'NOUN': 1, u'NONE': 2}

    def test_tokens_to_indices_uses_pos_tags(self):
        tokens = self.tokenizer.split_words(u"This is a sentence.")
        tokens = [t for t in tokens] + [Token(u"</S>")]
        vocab = Vocabulary()
        verb_index = vocab.add_token_to_namespace(u'VERB', namespace=u'pos_tags')
        cop_index = vocab.add_token_to_namespace(u'VBZ', namespace=u'pos_tags')
        none_index = vocab.add_token_to_namespace(u'NONE', namespace=u'pos_tags')
        # Have to add other tokens too, since we're calling `tokens_to_indices` on all of them
        vocab.add_token_to_namespace(u'DET', namespace=u'pos_tags')
        vocab.add_token_to_namespace(u'NOUN', namespace=u'pos_tags')
        vocab.add_token_to_namespace(u'PUNCT', namespace=u'pos_tags')

        indexer = PosTagIndexer(coarse_tags=True)

        indices = indexer.tokens_to_indices(tokens, vocab, u"tokens")
        assert len(indices) == 1
        assert u"tokens" in indices
        assert indices[u"tokens"][1] == verb_index
        assert indices[u"tokens"][-1] == none_index

        indexer._coarse_tags = False  # pylint: disable=protected-access
        assert indexer.tokens_to_indices([tokens[1]], vocab, u"coarse") == {u"coarse": [cop_index]}

    def test_padding_functions(self):
        indexer = PosTagIndexer()
        assert indexer.get_padding_token() == 0
        assert indexer.get_padding_lengths(0) == {}

    def test_as_array_produces_token_sequence(self):
        indexer = PosTagIndexer()
        padded_tokens = indexer.pad_token_sequence({u'key': [1, 2, 3, 4, 5]}, {u'key': 10}, {})
        assert padded_tokens == {u'key': [1, 2, 3, 4, 5, 0, 0, 0, 0, 0]}
